import os
import sys
import json
import tempfile
import shutil
import subprocess
from dektools.file import read_text, write_file
from dektools.sys import sys_paths_relative
from dektools.shell import shell_wrapper
from ..tmpl import ProjectGenerator
from .fixegg import bdist_fixegg

class bdist_fullegg(bdist_fixegg):
    description = "create an full-egg (egg with dependencies) distribution"

    def run(self):
        requirements = 'requirements.txt'
        if requirements and os.path.exists(requirements):
            self._install(requirements, self.bdist_dir)
        super().run()

    @staticmethod
    def _install(requirements, target):
        target = os.path.normpath(os.path.abspath(target))
        if shutil.which('pdm'):
            path_dir = tempfile.mkdtemp(prefix='dekegg-pdm-install')
            dependencies = []
            if requirements:
                for line in read_text(requirements).splitlines():
                    line = line.strip()
                    if line and not line.startswith('#'):
                        dependencies.append(line)
            ProjectGenerator(path_dir, dict(
                dependencies=json.dumps(dependencies)
            )).render()
            write_file(os.path.join(path_dir, 'pdm.lock'), mi='requirements.lock')
            last_dir = os.getcwd()
            os.chdir(path_dir)
            shell_wrapper('virtualenv .venv --no-pip --no-setuptools --no-wheel')
            shell_wrapper(f'pdm install')
            path_platlib = sys_paths_relative(os.path.join(path_dir, '.venv'))['platlib']
            for file in os.listdir(path_platlib):
                if file.endswith('.dist-info'):
                    p = os.path.join(path_platlib, file, 'REFER_TO')
                    if os.path.exists(p):
                        pp = os.path.join(read_text(p).strip(), 'lib')
                        if os.path.exists(pp):
                            for f in os.listdir(pp):
                                write_file(os.path.join(target, f), c=os.path.join(pp, f))
            shell_wrapper(f'{sys.executable} -m compileall {target}')
            os.chdir(last_dir)
        else:
            subprocess.check_call(
                [sys.executable, '-m', 'pip', 'install']
                + ['-U', '-t', target, '-r', requirements]
            )



from pathlib import Path
from dekgen.code.python.generator import CodeGenerator


class GeneratorBasic(CodeGenerator):
    TEMPLATE_DIR = Path(__file__).resolve().parent / 'templatefiles'

    def variables_data(self):
        return self.instance


class ProjectGenerator(GeneratorBasic):
    template_name = 'project'



import os
import re
from distutils import log
from setuptools.command.bdist_egg import bdist_egg, walk_egg, _get_purelib


class bdist_fixegg(bdist_egg):
    def remove_py(self, path):
        if path not in self.install_data_outputs:
            os.unlink(path)

    def do_install_data(self):
        # Hack for packages that install data to install's --install-lib
        self.get_finalized_command('install').install_lib = self.bdist_dir

        site_packages = os.path.normcase(os.path.realpath(_get_purelib()))
        old, self.distribution.data_files = self.distribution.data_files, []

        for item in old:
            if isinstance(item, tuple) and len(item) == 2:
                if os.path.isabs(item[0]):
                    realpath = os.path.realpath(item[0])
                    normalized = os.path.normcase(realpath)
                    if normalized == site_packages or normalized.startswith(
                            site_packages + os.sep
                    ):
                        item = realpath[len(site_packages) + 1:], item[1]
                        # XXX else: raise ???
            self.distribution.data_files.append(item)

        try:
            log.info("installing package data to %s", self.bdist_dir)
            cmd = self.call_command('install_data', force=0, root=None)
            self.install_data_outputs = set(cmd.get_outputs())
        finally:
            self.distribution.data_files = old

    def zap_pyfiles(self):
        log.info("Removing .py files from temporary directory")
        for base, dirs, files in walk_egg(self.bdist_dir):
            for name in files:
                path = os.path.join(base, name)

                if name.endswith('.py'):
                    log.debug("Deleting %s", path)
                    self.remove_py(path)

                if base.endswith('__pycache__'):
                    path_old = path

                    pattern = r'(?P<name>.+)\.(?P<magic>[^.]+)\.pyc'
                    m = re.match(pattern, name)
                    path_new = os.path.join(base, os.pardir, m.group('name') + '.pyc')
                    log.info("Renaming file from [%s] to [%s]" % (path_old, path_new))
                    try:
                        os.remove(path_new)
                    except OSError:
                        pass
                    os.rename(path_old, path_new)



import os
import sys
import glob
import tempfile
from dektools.file import normal_path, write_file, remove_path
from dektools.zip import decompress_files


def build_egg(path_project, bdist='bdist_egg', pyc=False):
    cwd = os.getcwd()
    os.chdir(path_project)
    path_out = tempfile.mkdtemp(prefix='dekegg-build_egg-out')
    os.system(' '.join(
        [
            sys.executable,
            "setup.py",
            "clean",
            "-a", bdist,  # bdist_egg , bdist_fullegg
            " --exclude-source-files" if pyc else "",
            "-d", path_out
        ]
    ))
    os.chdir(cwd)
    path_egg = glob.glob(os.path.join(path_out, "*.egg"))[0]
    path_egg = write_file(None, t=True, m=path_egg)
    remove_path(path_out)
    return path_egg


def is_egg(path_file):
    return os.path.splitext(path_file)[1].lower() == '.egg' and os.path.isfile(path_file)


def sure_egg_path(path):
    path = normal_path(path)
    if os.path.exists(path):
        return path
    cursor = path
    while True:
        if not is_egg(cursor):
            dirname = os.path.dirname(cursor)
            if dirname == cursor:
                break
            else:
                cursor = dirname
        else:
            return decompress_files(cursor) + path[len(cursor):]
    raise FileNotFoundError(path)
